@@ -1,6 +1,6 @@
 # imagecodecs/__init__.py
 
-__all__ = ()
+__all__ = []
 
 from .imagecodecs import __doc__, __version__, __getattr__, __dir__
 from .imagecodecs import _extensions
