@@ -1,8 +1,16 @@
 # imagecodecs/tests/conftest.py
 
+import os
+import sys
+
+if os.environ.get('VSCODE_CWD'):
+    # work around pytest not using PYTHONPATH in VSCode
+    sys.path.insert(
+        0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
+    )
+
 
 def pytest_report_header(config):
-    import sys
 
     try:
         pyversion = f'Python {sys.version.splitlines()[0]}'
